<?php
/*
 *  index.php
 *
 *  IRC-like WWW message board
 *  Tested under GNU/Linux, Apache 1.3.27 w/ PHP 4.3.0
 *
 *  version 0.1 (06.05.2003)
 *  by Michal Miszewski <fixer@irc.pl>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation.
*/

/* configuration */
$name = "Message Board";
$logfilename = "chat.csv";
$passwdfilename = "passwd.csv";
$charset = "iso-8859-1";
$nicklen = 8;
$passwdlen = 16;
$msglen = 256;
$nummsg = 16;
$membersonly = 1;

?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
<head>
  <meta http-equiv="Content-Type" content="text/html; charset=<?php echo "$charset"; ?>" />
  <meta name="author" content="Michal Miszewski" />
  <meta name="description" content="<?php echo "$name"; ?>" />
  <link rel="stylesheet" title="Default" href="style.css" type="text/css" />
  <title><?php echo "$name"; ?></title>
</head>
<body>

<h1><?php echo "$name"; ?></h1>
<div id="chat">
<?php
  if ((strlen($_REQUEST["post"]) > 0) && (strlen($_REQUEST["msg"]) > 0))
  {
    if (strlen($_REQUEST["nick"]) > $nicklen) { $error = $error . "Nick too long. "; }
    if (strlen($_REQUEST["nick"]) == 0) { $error = $error . "Must supply a nickname. "; }
    if (strlen($_REQUEST["msg"]) > $msglen) { $error = $error . "Message too long. "; }
    if (strlen($_REQUEST["passwd"]) > $passwdlen) { $error = $error . "Invalid password. "; }
    if (strlen($error) == 0)
    {
      /* password check */
      if (is_readable("$passwdfilename") != true) { exit("Critical: Can't read users database!\n"); }
      $file = fopen("$passwdfilename", 'r');
      $ok = 0;
      $reg = 0;
      do
      {
        $line = fgetcsv($file, 300);
	if ((strlen($line[0]) > 0) && (strlen($line[1]) > 0))
	{
	  if (strcmp(strtolower($line[0]), strtolower($_REQUEST["nick"])) == 0)
	  {
	    $reg = 1;
	    if (strlen($_REQUEST["passwd"]) == 0)
	    { $error = $error . "Nick is registered; enter password."; }
	    else
	    {
	      if (strcmp($line[1], $_REQUEST["passwd"]) == 0) { $ok = 1; }
	      else { $error = $error . "Incorrect password."; }
	    }
	  }
	}
      }
      while((feof($file) == false) && ($ok == 0));
      fclose($file);
      /* message posting */
      if (($ok == 1) || ($reg == 0))
      {
        if (($reg == 0) && ($membersonly == 1)) { $error = $error . "Sorry, this board is for registered users only."; }
	else
	{
	  if (is_writeable("$logfilename") != true) { exit("Critical: Can't write message to file!\n"); }
          $date = date("dmYHi", time());
          $file = fopen("$logfilename", 'a');
          $nick = $_REQUEST["nick"];
	  if ($reg == 0) $nick = "~" . $nick;
          $msg = $_REQUEST["msg"];
	  flock($file, 1);
          fputs($file, "$date, $nick, \"$msg\"\n");
	  flock($file, 3);
          fclose($file);
	}
      }
    }
  }
  /* listing of messages */
  if (is_readable("$logfilename") != true) { exit("Critical: Can't read messages from file!\n"); }
  $file = fopen("$logfilename", 'r');
  while (fgets($file, 512)) $num++;  /* number of lines in the file */
  rewind($file);
  if ($nummsg < $num)
    while ($n < ($num - $nummsg)) { fgets($file, 512); $n++; }
  do
  {
    $line = fgetcsv($file, 1000);
    if ((strcmp($line[0], "") != 0) && (strcmp($line[1], "") != 0))
    {
      /* formatting */
      $date = substr($line[0], 0, 2) . "-" . substr($line[0], 2, 2) . "-" . substr($line[0], 4, 4);
      $time = substr($line[0], 8, 2) . ":" . substr($line[0], 10, 2);
      $nick = $line[1];
      $msg = htmlentities($line[2]);
      echo "$date $time &lt;<b>$nick</b>&gt; $msg";
      echo "<br />\n";
    }
  }
  while(feof($file) == false);
  fclose($file);
?>
</div>

<?php
  if (strlen($error) > 0)
  {
    echo "<div id=\"error\">";
    echo "<b>Error:</b> $error";
    echo "</div>";
  }
?>

<div id="input">
<form method="post" action="">
<table>
  <tr>
  <td>Nick:</td>
  <td>
  <input type="hidden" name="post" value="yes" />
  <input type="text" name="nick" value="<?php echo $_REQUEST["nick"]; ?>" size="16" maxlength="<?php echo $nicklen; ?>" />
  (password: <input type="password" name="passwd" value="<?php echo $_REQUEST["passwd"]; ?>" size="16" maxlength="<?php echo $passwdlen; ?>" />)
  </td>
  </tr>
  <tr>
  <td>Message:</td>
  <td><input type="text" name="msg" value="" size="100" maxlength="<?php echo $msglen; ?>" /></td>
  </tr>
</table>
  <p><input type="submit" value="Send/Refresh" /></p>
</form>
</div>

<div id="foot">
  Created by <a href="http://fixer.qwe.pl/">Michal Miszewski</a>.
  Valid <a href="http://www.w3.org/TR/2000/REC-xhtml1-20000126/">XHTML 1.0</a>
  and <a href="http://www.w3.org/TR/REC-CSS2/">CSS</a>.
</div>

</body>
</html>
